@@ -1,17 +1,9 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from ..model import ModelSQL, fields
-from .. import backend
-from ..pool import Pool, PoolMeta
-from ..transaction import Transaction
-
-__all__ = [
-    'UIMenuGroup', 'ActionGroup', 'ModelFieldGroup', 'ModelButtonGroup',
-    'ModelButtonRule', 'ModelButtonClick',
-    'RuleGroupGroup', 'RuleGroupUser', 'Lang', 'SequenceType',
-    'SequenceTypeGroup', 'Sequence', 'SequenceStrict',
-    'ModuleConfigWizardItem',
-    ]
+from trytond.model import ModelSQL, DeactivableMixin, fields
+from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Eval
+from trytond.transaction import Transaction
 
 
 class UIMenuGroup(ModelSQL):
@@ -22,20 +14,6 @@ class UIMenuGroup(ModelSQL):
     group = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=True, required=True)
 
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        transaction = Transaction()
-        # Migration from 1.0 table name change
-        TableHandler.table_rename('ir_ui_menu_group_rel', cls._table)
-        transaction.database.sequence_rename(transaction.connection,
-            'ir_ui_menu_group_rel_id_seq', cls._table + '_id_seq')
-        # Migration from 2.0 menu_id and gid renamed into menu group
-        table = TableHandler(cls, module_name)
-        table.column_rename('menu_id', 'menu')
-        table.column_rename('gid', 'group')
-        super(UIMenuGroup, cls).__register__(module_name)
-
     @classmethod
     def create(cls, vlist):
         res = super(UIMenuGroup, cls).create(vlist)
@@ -64,20 +42,6 @@ class ActionGroup(ModelSQL):
     group = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=True, required=True)
 
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        transaction = Transaction()
-        # Migration from 1.0 table name change
-        TableHandler.table_rename('ir_action_group_rel', cls._table)
-        transaction.database.sequence_rename(transaction.connection,
-            'ir_action_group_rel_id_seq', cls._table + '_id_seq')
-        # Migration from 2.0 action_id and gid renamed into action and group
-        table = TableHandler(cls, module_name)
-        table.column_rename('action_id', 'action')
-        table.column_rename('gid', 'group')
-        super(ActionGroup, cls).__register__(module_name)
-
     @classmethod
     def create(cls, vlist):
         Action = Pool().get('ir.action')
@@ -111,42 +75,66 @@ class ActionGroup(ModelSQL):
         Pool().get('ir.rule')._domain_get_cache.clear()
 
 
-class ModelFieldGroup(ModelSQL):
-    "Model Field Group Rel"
-    __name__ = 'ir.model.field-res.group'
-    field = fields.Many2One('ir.model.field', 'Model Field',
-            ondelete='CASCADE', select=True, required=True)
-    group = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
-            select=True, required=True)
+class Action(metaclass=PoolMeta):
+    __name__ = 'ir.action'
+
+    groups = fields.Many2Many(
+        'ir.action-res.group', 'action', 'group', "Groups")
+
+
+class ActionMixin(metaclass=PoolMeta):
 
     @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        database = Transaction().database
-        transaction = Transaction()
-        # Migration from 1.0 table name change
-        TableHandler.table_rename('ir_model_field_group_rel', cls._table)
-        transaction.database.sequence_rename(transaction.connection,
-            'ir_model_field_group_rel_id_seq', cls._table + '_id_seq')
-        table = TableHandler(cls, module_name)
-        # Migration from 2.6: field_id and group_id renamed to field and group
-        table.column_rename('field_id', 'field')
-        table.column_rename('group_id', 'group')
-        super(ModelFieldGroup, cls).__register__(module_name)
+    def get_groups(cls, name, action_id=None):
+        with Transaction().set_context(_check_access=False):
+            # TODO add cache
+            domain = [
+                (cls._action_name, '=', name),
+                ]
+            if action_id:
+                domain.append(('id', '=', action_id))
+            actions = cls.search(domain)
+            groups = {g.id for a in actions for g in a.groups}
+            return groups
 
 
-class ModelButtonGroup(ModelSQL):
+class ActionReport(ActionMixin):
+    __name__ = 'ir.action.report'
+
+
+class ActionActWindow(ActionMixin):
+    __name__ = 'ir.action.act_window'
+
+
+class ActionWizard(ActionMixin):
+    __name__ = 'ir.action.wizard'
+
+
+class ActionURL(ActionMixin):
+    __name__ = 'ir.action.url'
+
+
+class ActionKeyword(metaclass=PoolMeta):
+    __name__ = 'ir.action.keyword'
+
+    groups = fields.Function(fields.One2Many('res.group', None, 'Groups'),
+        'get_groups', searcher='search_groups')
+
+    def get_groups(self, name):
+        return [g.id for g in self.action.groups]
+
+    @classmethod
+    def search_groups(cls, name, clause):
+        return [('action.' + clause[0],) + tuple(clause[1:])]
+
+
+class ModelButtonGroup(DeactivableMixin, ModelSQL):
     "Model Button - Group"
     __name__ = 'ir.model.button-res.group'
     button = fields.Many2One('ir.model.button', 'Button',
         ondelete='CASCADE', select=True, required=True)
     group = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
         select=True, required=True)
-    active = fields.Boolean('Active', select=True)
-
-    @staticmethod
-    def default_active():
-        return True
 
     @classmethod
     def create(cls, vlist):
@@ -171,14 +159,12 @@ class ModelButtonGroup(ModelSQL):
         pool.get('ir.model.button')._groups_cache.clear()
 
 
-class ModelButtonRule:
-    __metaclass__ = PoolMeta
+class ModelButtonRule(metaclass=PoolMeta):
     __name__ = 'ir.model.button.rule'
     group = fields.Many2One('res.group', "Group", ondelete='CASCADE')
 
 
-class ModelButtonClick:
-    __metaclass__ = PoolMeta
+class ModelButtonClick(metaclass=PoolMeta):
     __name__ = 'ir.model.button.click'
     user = fields.Many2One('res.user', "User", ondelete='CASCADE')
 
@@ -191,48 +177,8 @@ class RuleGroupGroup(ModelSQL):
     group = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=True, required=True)
 
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        transaction = Transaction()
-        # Migration from 1.0 table name change
-        TableHandler.table_rename('group_rule_group_rel', cls._table)
-        transaction.database.sequence_rename(transaction.connection,
-            'group_rule_group_rel_id_seq', cls._table + '_id_seq')
-        # Migration from 2.0 rule_group_id and group_id renamed into rule_group
-        # and group
-        table = TableHandler(cls, module_name)
-        table.column_rename('rule_group_id', 'rule_group')
-        table.column_rename('group_id', 'group')
-        super(RuleGroupGroup, cls).__register__(module_name)
 
-
-class RuleGroupUser(ModelSQL):
-    "Rule Group - User"
-    __name__ = 'ir.rule.group-res.user'
-    rule_group = fields.Many2One('ir.rule.group', 'Rule Group',
-            ondelete='CASCADE', select=True, required=True)
-    user = fields.Many2One('res.user', 'User', ondelete='CASCADE',
-            select=True, required=True)
-
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        transaction = Transaction()
-        # Migration from 1.0 table name change
-        TableHandler.table_rename('user_rule_group_rel', cls._table)
-        transaction.database.sequence_rename(transaction.connection,
-            'user_rule_group_rel_id_seq', cls._table + '_id_seq')
-        # Migration from 2.0 rule_group_id and user_id renamed into rule_group
-        # and user
-        table = TableHandler(cls, module_name)
-        table.column_rename('rule_group_id', 'rule_group')
-        table.column_rename('user_id', 'user')
-        super(RuleGroupUser, cls).__register__(module_name)
-
-
-class Lang:
-    __metaclass__ = PoolMeta
+class Lang(metaclass=PoolMeta):
     __name__ = 'ir.lang'
 
     @classmethod
@@ -242,12 +188,11 @@ class Lang:
         Pool().get('res.user')._get_preferences_cache.clear()
 
 
-class SequenceType:
-    __metaclass__ = PoolMeta
+class SequenceType(metaclass=PoolMeta):
     __name__ = 'ir.sequence.type'
     groups = fields.Many2Many('ir.sequence.type-res.group', 'sequence_type',
             'group', 'User Groups',
-            help='Groups allowed to edit the sequences of this type')
+            help='Groups allowed to edit the sequences of this type.')
 
 
 class SequenceTypeGroup(ModelSQL):
@@ -281,47 +226,7 @@ class SequenceTypeGroup(ModelSQL):
         Rule._domain_get_cache.clear()
 
 
-class Sequence:
-    __metaclass__ = PoolMeta
-    __name__ = 'ir.sequence'
-    groups = fields.Function(fields.Many2Many('res.group', None, None,
-        'User Groups'), 'get_groups', searcher='search_groups')
-
-    @classmethod
-    def get_groups(cls, sequences, name):
-        SequenceType = Pool().get('ir.sequence.type')
-        code2seq = {}
-        for sequence in sequences:
-            code2seq.setdefault(sequence.code, []).append(sequence.id)
-
-        sequence_types = SequenceType.search([
-                ('code', 'in', code2seq.keys()),
-                ])
-
-        groups = {}
-        for sequence_type in sequence_types:
-            seq_ids = code2seq[sequence_type.code]
-            for seq_id in seq_ids:
-                groups.setdefault(seq_id, []).append(sequence_type.id)
-
-        return groups
-
-    @staticmethod
-    def search_groups(name, clause):
-        SequenceType = Pool().get('ir.sequence.type')
-        seq_types = SequenceType.search([clause], order=[])
-        codes = set(st.code for st in seq_types)
-        return [('code', 'in', list(codes))]
-
-
-class SequenceStrict(Sequence):
-    # This empty class declaration is needed to inherit the groups
-    # field
-    __name__ = 'ir.sequence.strict'
-
-
-class ModuleConfigWizardItem:
-    __metaclass__ = PoolMeta
+class ModuleConfigWizardItem(metaclass=PoolMeta):
     __name__ = 'ir.module.config_wizard.item'
 
     @classmethod
@@ -348,3 +253,38 @@ class ModuleConfigWizardItem:
         super(ModuleConfigWizardItem, cls).delete(items)
         # Restart the cache for get_preferences
         User._get_preferences_cache.clear()
+
+
+class Export(metaclass=PoolMeta):
+    __name__ = 'ir.export'
+
+    groups = fields.Many2Many(
+        'ir.export-res.group', 'export', 'group', "Groups",
+        help="The user groups that can use the export.")
+    write_groups = fields.Many2Many(
+        'ir.export-write-res.group', 'export', 'group',
+        "Modification Groups",
+        domain=[
+            ('id', 'in', Eval('groups', [])),
+            ],
+        states={
+            'invisible': ~Eval('groups'),
+            },
+        depends=['groups'],
+        help="The user groups that can modify the export.")
+
+
+class Export_Group(ModelSQL):
+    "Export Group"
+    __name__ = 'ir.export-res.group'
+
+    export = fields.Many2One(
+        'ir.export', "Export", required=True, select=True, ondelete='CASCADE')
+    group = fields.Many2One(
+        'res.group', "Group", required=True, ondelete='CASCADE')
+
+
+class Export_Write_Group(Export_Group):
+    "Export Modification Group"
+    __name__ = 'ir.export-write-res.group'
+    _table = None  # Needed to reset Export_Group._table
